from builtins import object
from crypt import crypt
from string import ascii_letters, digits
from random import choice
import subprocess
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict


class UserExists(Exception):

    def __str__(self):
        return "User already exists"


class UserNotExists(Exception):

    def __str__(self):
        return "User not exists"


class UnknownEncryptionMode(Exception):

    def __init__(self, mode):
        self.mode = mode

    def __str__(self):
        return "Encryption Mode %s is unknown/unsupported" % self.mode


class Basic(object):

    """ Basic object deals with Basic HTTP Authorization configuration file.
    It is passed the path to userdb file. """

    def __init__(self, userdb, mode="crypt"):
        self.encryption_mode = mode
        self.userdb = userdb
        self.initial_users = OrderedDict()
        self.new_users = OrderedDict()

    def __enter__(self):
        with open(self.userdb, "r") as users:
            for i in users:
                user, password = i.split(":", 1)
                self.initial_users[user] = password
        self.new_users = self.initial_users.copy()
        return self

    def __exit__(self, type, value, traceback):
        if self.new_users == self.initial_users:
            return
        with open(self.userdb, "w") as userdb:
            for user in self.new_users:
                userdb.write("%s:%s" % (user, self.new_users[user]))

    def __contains__(self, user):
        return user in self.users

    @property
    def users(self):
        """ Returns users in a tuple """
        return list(self.new_users.keys())

    def add(self, user, password):
        """ Adds a user with password """
        if self.__contains__(user):
            raise UserExists
        self.new_users[user] = self._encrypt_password(password) + "\n"

    def pop(self, user):
        """ Deletes a user """
        if not self.__contains__(user):
            raise UserNotExists
        self.new_users.pop(user)

    def change_password(self, user, password):
        """ Changes user password """
        if not self.__contains__(user):
            raise UserNotExists
        self.new_users[user] = self._encrypt_password(password) + "\n"

    def _encrypt_password(self, password):
        """encrypt the password for given mode """
        if self.encryption_mode.lower() == 'crypt':
            return self._crypt_password(password)
        elif self.encryption_mode.lower() == 'md5':
            return self._md5_password(password)
        else:
            raise UnknownEncryptionMode(self.encryption_mode)

    def _crypt_password(self, password):
        """ Crypts password """

        def salt():
            """ Generates some salt """
            symbols = ascii_letters + digits
            return choice(symbols) + choice(symbols)

        return crypt(password, salt())

    def _md5_password(self, password):
        """ Crypts password using openssl binary and MD5 encryption """
        return subprocess.check_output(['openssl', 'passwd', '-apr1', password]).decode('utf-8').strip()



from builtins import object
import re
from orderedmultidict import omdict


class GroupNotExists(Exception):

    def __str__(self):
        return "Group not exists"


class UserAlreadyInAGroup(Exception):

    def __str__(self):
        return "User already in a group"


class UserNotInAGroup(Exception):

    def __str__(self):
        return "User not in a group"


class Group(object):

    """ Group object deals with group authorization files.  It is passed the
    path to groupdb file. """

    def __init__(self, groupdb):
        self.groupdb = groupdb
        self.initial_groups = omdict()
        self.new_groups = omdict()

    def __enter__(self):
        with open(self.groupdb, "r") as groupdb:
            groupdb = re.sub("\\\\\n", "", groupdb.read())
            for group in groupdb.splitlines():
                groupname, users = group.split(": ", 1)
                for user in users.split():
                    self.initial_groups.add(groupname, user)
            self.new_groups = self.initial_groups.copy()
            return self

    def __exit__(self, type, value, traceback):
        if self.new_groups == self.initial_groups:
            return
        with open(self.groupdb, "w") as userdb:
            for group in self.new_groups:
                userdb.write("%s: %s\n" % (group, " ".join(self.new_groups.getlist(group))))

    def __contains__(self, group):
        return group in self.groups

    @property
    def groups(self):
        """ Returns groups in a tuple """
        return list(self.new_groups.keys())

    def is_user_in(self, user, group):
        """ Returns True if user is in a group """
        return user in self.new_groups.getlist(group)

    def add_user(self, user, group):
        """ Adds user to a group """
        if self.is_user_in(user, group):
            raise UserAlreadyInAGroup
        self.new_groups.add(group, user)

    def delete_user(self, user, group):
        """ Deletes user from group """
        if not self.__contains__(group):
            raise GroupNotExists
        if not self.is_user_in(user, group):
            raise UserNotInAGroup
        self.new_groups.popvalue(group, user)



#!/usr/bin/env python

from sys import version_info

from setuptools import setup, find_packages

python_version = "%s.%s" % (version_info[0], version_info[1])

if python_version == "2.7":
    requires = ["orderedmultidict>=0.7", "future"]
else:
    requires = ["orderedmultidict>=0.7"]

setup(
    name="htpasswd",
    version="2.3",
    packages=["htpasswd"],
    install_requires=requires,
    author="Ilya Otyutskiy",
    author_email="ilya.otyutskiy@icloud.com",
    maintainer="Ilya Otyutskiy",
    url="https://github.com/thesharp/htpasswd",
    description="Library to work with htpasswd user (basic authorization) and group files.",
    license="MIT"
)
